#include "Contact.h"
#include "Controller.h"

ScreenNameType _snType;
NSArray* _afTypes;

id mutablize(id obj)
{
  id m, n; 
  NSEnumerator *e;
  
  if([obj isKindOf: [NSDictionary class]])
    {
      m = [NSMutableDictionary dictionaryWithCapacity: [obj count]];
      e = [obj keyEnumerator];
      while((n = [e nextObject]))
	[m setObject: mutablize([obj objectForKey: n]) forKey: n];
      return m;
    }
  else if([obj isKindOf: [NSArray class]])
    {
      m = [NSMutableArray arrayWithCapacity: [obj count]];
      e = [obj objectEnumerator];
      while((n = [e nextObject]))
	[m addObject: mutablize(n)];
      return m;
    }

  return obj;
}

@implementation Contact
+ (void) initialize
{
  _snType = ScreenNameBeginsWithLastName;
  _afTypes = [[NSArray alloc]
	       initWithObjects: @"POBox", @"ExtendedAddress",
	       @"Street", @"City", @"State", @"ZipCode", @"Country",
	       nil];
}

+ (ScreenNameType) screenNameType
{
  return _snType;
}
  
+ (void) setScreenNameType: (ScreenNameType) type
{
  _snType = type;
}

+ (int) addressIndexForAddressField: (NSString*) field
{
  return [_afTypes indexOfObject: field];
}

+ (NSString*) addressFieldForAddressIndex: (int) index;
{
  return [_afTypes objectAtIndex: index];
}

- init
{
  NSAssert(NO, @"Cannot init without ID");
  return nil;
}

- initWithId: (int) i
{
  [super init];
  NSAssert(i > 0, @"Not a valid ID");
  _id = i;
  _info = nil;
  return self;
}

- initWithDictionary: (NSDictionary*) dict
		  id: (int) i
{
  [super init];
  NSAssert(i > 0, @"Not a valid ID");
  _id = i;
  _info = [dict retain];
  return self;
}

- initWithContentsOfFile: (NSString*) filename
{
  [super init];
  _id = [[filename lastPathComponent] intValue];
  NSAssert(_id > 0, @"Not a valid ID");

  NSDictionary *dict = [[NSDictionary alloc] initWithContentsOfFile: filename];
  [self setDictionary: dict];

  _dir = [[filename stringByDeletingLastPathComponent] retain];

  return self;
}

- (void) dealloc
{
  [_info release];
  [super dealloc];
}

- (void) setDictionary: (NSDictionary*) dict
{
  if(_info) [_info release];
  _info = [mutablize(dict) retain];
}

- (NSDictionary*) dictionary
{
  return _info;
}

- (void) writeToFile
{
  if(!_dir)
    _dir = [[Controller databaseRoot] retain];
  [self writeToFileInDirectory: _dir];
}

- (void) writeToFileInDirectory: (NSString*) directory
{
  if(![directory isEqualToString: _dir])
    {
      [_dir release];
      _dir = [directory retain];
    }
  [_info writeToFile: [_dir stringByAppendingPathComponent: [self fileName]]
	 atomically: YES];
}

- (int) getId
{
  return _id;
}

- (NSArray*) valuesForKey: (NSString*) key
		parameter: (id) parameter
{
  int i, j;
  NSArray *arr = [_info objectForKey: key];
  if(!arr) return nil;
  if(![arr count]) return nil;

  if(!parameter)
    return [[arr objectAtIndex: 0] objectForKey: @"Values"];
  else if([parameter isKindOfClass: [NSString class]])
    {
      for(i=0; i<[arr count]; i++)
	if([[[arr objectAtIndex: i] objectForKey: @"Parameters"]
	     containsObject: parameter])
	  return [[arr objectAtIndex: i] objectForKey: @"Values"];
    }
  else if([parameter isKindOfClass: [NSArray class]])
    {
      for(i=0; i<[arr count]; i++)
	{
	  for(j=0; j<[parameter count]; j++)
	    {
	      if(![[[arr objectAtIndex: i] objectForKey: @"Parameters"]
		    containsObject: [parameter objectAtIndex: j]])
		break;
	    }
	  if(j == [parameter count])
	    return [[arr objectAtIndex: i] objectForKey: @"Values"];
	}
      return nil;
    }

  NSLog(@"Unknown parameter type %@\n", [parameter className]);
  return nil;
}

- (NSString*) valueForKey: (NSString*) key
		parameter: (id) parameter
		    index: (int) index
{
  NSArray *arr = [self valuesForKey: key parameter: parameter];
  if(!arr) return nil;
  if([arr count] <= index) return nil;
  return [arr objectAtIndex: index];
}

- (BOOL) _valuesEmpty: (NSArray*) values
{
  NSEnumerator *e; NSString *s; BOOL allEmpty;

  if(!values) return YES;
  if(![values count]) return YES;

  allEmpty = YES;
  e = [values objectEnumerator];
  while((s = [e nextObject]))
    if(![s isEqualToString: @""])
      {
	allEmpty = NO;
	break;
      }
  if(allEmpty) return YES;
  return NO;
}

- (void) deleteValuesForKey: (NSString*) key
		  parameter: (id) parameter
{
  NSLog(@"Deleting for %@ params %@\n",
	[key description], [parameter description]);

  int i, j;

  if(!parameter)
    {
      [_info removeObjectForKey: key];
      return;
    }
  
  NSMutableArray *arr = [_info objectForKey: key];
  if(!arr) return;
  if(![arr count]) return;
  
  if([parameter isKindOfClass: [NSString class]])
    {
      for(i=0; i<[arr count]; i++)
	if([[[arr objectAtIndex: i] objectForKey: @"Parameters"]
	     containsObject: parameter])
	  {
	    [arr removeObjectAtIndex: i--];
	  }
    }
  else if([parameter isKindOfClass: [NSArray class]])
    {
      for(i=0; i<[arr count]; i++)
	{
	  for(j=0; j<[parameter count]; j++)
	    {
	      if(![[[arr objectAtIndex: i] objectForKey: @"Parameters"]
		    containsObject: [parameter objectAtIndex: j]])
		break;
	    }
	  if(j == [parameter count])
	    [arr removeObjectAtIndex: i--];
	}
    }
  else
    NSLog(@"Unknown parameter type %@\n", [parameter className]);
}

- (void) deleteValuesForKey: (NSString*) key
		  parameter: (id) parameter
		    atIndex: (int) index
{
  NSLog(@"Deleting for %@ params %@ index %d\n",
	[key description], [parameter description], index);
  NSArray *values = [self valuesForKey: key parameter: parameter];

  // not in anyway?
  if(!values) return;
  if(index >= [values count]) return;

  // only element? Remove entire values array
  if(index == 0 && [values count] == 1)
    return [self deleteValuesForKey: key parameter: parameter];

  // last element? Truncate existing values array
  if(index == [values count]-1)
    {
      values = [values subarrayWithRange: NSMakeRange(0, [values count]-1)];
      return [self setValues: values
		   forKey: key
		   parameter: parameter];
    }

  // set element to the empty string
  NSMutableArray *arr = [NSMutableArray arrayWithArray: values];
  [arr replaceObjectAtIndex: index withObject: @""];
  return [self setValues: arr
	       forKey: key
	       parameter: parameter];  
}

- (void) setValues: (NSArray*) values
	    forKey: (NSString*) key
	 parameter: (id) parameter
{
  int i, j;

  if([self _valuesEmpty: values])
    return [self deleteValuesForKey: key
		 parameter: parameter];
  
  NSMutableArray *arr = [_info objectForKey: key];

  // don't have anything for this key? Add it.
  if(!arr)
    {
      arr = [NSMutableArray arrayWithCapacity: 1];
      NSMutableDictionary *dict =
	[NSMutableDictionary dictionaryWithCapacity: 1];
      [dict setObject: values forKey: @"Values"];
      [dict setObject: [[NSDate date] description] forKey: @"Timestamp"];
      [arr addObject: dict];
      [_info setObject: arr forKey: key];
      return;
    }
  
  if(!parameter)
    {
      [[arr objectAtIndex: 0] setObject: values forKey: @"Values"];
      [[arr objectAtIndex: 0]
	setObject: [[NSDate date] description] forKey: @"Timestamp"];
      return;
    }

  if([parameter isKindOfClass: [NSString class]])
    {
      for(i=0; i<[arr count]; i++)
	if([[[arr objectAtIndex: i] objectForKey: @"Parameters"]
	     containsObject: parameter])
	  {
	    [[arr objectAtIndex: i] setObject: values forKey: @"Values"];
	    return;
	  }
      
      NSMutableDictionary *dict =
	[NSMutableDictionary dictionaryWithCapacity: 1];
      [dict setObject: values forKey: @"Values"];
      [dict setObject: [[NSDate date] description] forKey: @"Timestamp"];
      [dict setObject: [NSMutableArray arrayWithObjects: parameter, nil]
	    forKey: @"Parameters"];
      [arr addObject: dict];
      return;
    }

  if([parameter isKindOfClass: [NSArray class]])
    {
      for(i=0; i<[arr count]; i++)
	{
	  for(j=0; j<[parameter count]; j++)
	    {
	      if(![[[arr objectAtIndex: i] objectForKey: @"Parameters"]
		    containsObject: [parameter objectAtIndex: j]])
		break;
	    }
	  if(j == [parameter count])
	    {
	      [[arr objectAtIndex: i] setObject: values forKey: @"Values"];
	      return;
	    }
	}
      NSMutableDictionary *dict =
	[NSMutableDictionary dictionaryWithCapacity: 1];
      [dict setObject: values forKey: @"Values"];
      [dict setObject: [[NSDate date] description] forKey: @"Timestamp"];
      
      NSMutableArray *p =
	[NSMutableArray arrayWithCapacity: [parameter count]];
      for(j=0; j<[parameter count]; j++)
	[p addObject: [parameter objectAtIndex: j]];
      [dict setObject: p forKey: @"Parameters"];
      [arr addObject: dict];
      return;
    }
}

- (void) setValue: (NSString*) value
	   forKey: (NSString*) key
	parameter: (id) parameter
	  atIndex: (int) index
{
  int i;

  if([value isEqualToString: @""])
    return [self deleteValuesForKey: key
		 parameter: parameter
		 atIndex: index];
  
  NSMutableArray *arr = [self valuesForKey: key parameter: parameter];

  if(!arr)
    {
      NSMutableArray *v = [NSMutableArray arrayWithCapacity: index+1];
      for(i=0; i<index-1; i++)
	[v addObject: @""];
      [v addObject: value];
      [self setValues: v forKey: key parameter: parameter];
      return;
    }

  // pad with nil entries if necessary
  for(i=[arr count]; i<=index; i++)
    [arr addObject: @""];
  [arr replaceObjectAtIndex: index withObject: value];
  [self setValues: [NSMutableArray arrayWithArray: arr]
	forKey: key
	parameter: parameter];
}

- (NSString*) screenName
{
  if([[self class] screenNameType] == ScreenNameBeginsWithLastName)
    return [NSString stringWithFormat: @"%@, %@",
		     [self lastName], [self firstName]];
  else
    return [NSString stringWithFormat: @"%@ %@",
		     [self firstName], [self lastName]];
}

- (int) screenNameCompare: (Contact*) anotherContact
{
  return [[self screenName]
	   caseInsensitiveCompare: [anotherContact screenName]];
}

- (NSString*) firstName
{
  return [self valueForKey: @"n" parameter: nil index: 1];
}  

- (NSString*) lastName
{
  return [self valueForKey: @"n" parameter: nil index: 0];
}

- (NSString*) title
{
  NSArray *arr = [self valuesForKey: @"title" parameter: nil];
  return [arr objectAtIndex: 0];
}

- (NSString*) email
{
  NSArray *arr = [self valuesForKey: @"email" parameter: nil];
  return [arr objectAtIndex: 0];
}

- (NSString*) phone
{
  NSArray *arr = [self valuesForKey: @"tel" parameter: nil];
  return [arr objectAtIndex: 0];
}

- (NSImage*) image
{
  NSArray *arr = [self valuesForKey: @"photo" parameter: nil];
  if(!arr) return nil;
  return [[[NSImage alloc] initWithData: [arr objectAtIndex: 0]]
	   autorelease];
}

- (NSString*) addressRoad
{
  NSArray *arr = [self valuesForKey: @"adr" parameter: nil];
  return [arr objectAtIndex: 2];
}

- (NSString*) addressPostalCode
{
  NSArray *arr = [self valuesForKey: @"adr" parameter: nil];
  return [arr objectAtIndex: 5];
}

- (NSString*) addressState;
{
  NSArray *arr = [self valuesForKey: @"adr" parameter: nil];
  return [arr objectAtIndex: 6];
}

- (NSString*) addressCity;
{
  NSArray *arr = [self valuesForKey: @"adr" parameter: nil];
  return [arr objectAtIndex: 3];
}

- (NSString*) fileName
{
  return [NSString stringWithFormat: @"%d", _id];
}

- (void) setImage: (NSImage*) image
{
  NSMutableDictionary *dict;
  NSMutableArray *arr = [_info objectForKey: @"photo"];
  if(!arr)
    arr = [NSMutableArray arrayWithCapacity: 2];
  if([arr count])
    dict = [arr objectAtIndex: 0];
  else
    dict = [NSMutableDictionary dictionaryWithCapacity: 2];
  
  [dict setObject: [NSMutableArray
		     arrayWithObjects: [image TIFFRepresentation],
		     nil]
	forKey: @"Values"];
  [dict setObject: [[NSDate date] description]
	forKey: @"Timestamp"];
  if([arr count])
    [arr replaceObjectAtIndex: 0 withObject: dict];
  else
    [arr addObject: dict];
  [_info setObject: arr forKey: @"photo"];
  [self writeToFile];
}

- (BOOL) takeImageFromFile: (NSString*) filename
{
  NSImage *img = [[[NSImage alloc]
		    initWithContentsOfFile: filename]
		   autorelease];
  if(!img) return NO;
  [self setImage: img];
  return YES;
}

@end
