// ContactView.m (this is -*- ObjC -*-)
// 
// \author: Bjrn Giesler <giesler@ira.uka.de>
// 
// 
// 
// $Author: bjoern $
// $Locker:  $
// $Revision: 1.1.1.1 $
// $Date: 2003/08/28 15:54:43 $

#include "ContactView.h"
#include "ContactViewDisplayLayouter.h"
#include "ContactViewEditLayouter.h"

@implementation ContactView

- initWithFrame: (NSRect) frameRect
{
  [super initWithFrame: frameRect];

  allCells = nil;
  textObject = nil;
  editingCell = nil;

  [[NSNotificationCenter defaultCenter]
    addObserver: self
    selector: @selector(superviewFrameChanged:)
    name: NSViewFrameDidChangeNotification
    object: nil];
  
  return self;
}

- (void) dealloc
{
  [labels release];
  [firstNameCell release];
  [lastNameCell release];
  [titleCell release];
  [photoCell release];
  [contact release];

  [allCells release];
}

- (void) layout
{
  if(!contact) return;

  [allCells release];
  contentSize = NSZeroSize;
  
  if(editable)
    allCells = [[ContactViewEditLayouter doLayoutForContact: contact
					 needsSize: &contentSize
					 target: self]
		 retain];
  else
    allCells = [[ContactViewDisplayLayouter doLayoutForContact: contact
					    needsSize: &contentSize
					    target: self]
		   retain];
  [self calcSize];
  [self setNeedsDisplay: YES];
}

- (BOOL) fillsSuperview
{
  return fillsSuperview;
}
- (void) setFillsSuperview: (BOOL) yesno
{
  fillsSuperview = yesno;
  [self calcSize];
}

- (void) calcSize
{
  NSRect r;

  r.origin = NSMakePoint(0, 0);
  r.size = contentSize;

  if(fillsSuperview)
    {
      NSSize s;
      s = [[self superview] frame].size;
      if(r.size.height < s.height)
	{
	  NSEnumerator *e; id cell;
	  e = [allCells objectEnumerator];
	  while((cell = [e nextObject]))
	    [cell moveDownBy: r.size.height - s.height];
	  r.size.height = s.height;
	}
      
      if(r.size.width < s.width)
	r.size.width = s.width;
    }

  selfResized = YES;
  [self setFrame: r];
  [self setNeedsDisplay: YES];
}

- (void) setContact: (Contact*) anObject
{
  if(anObject == contact) return;
  
  [contact release];
  contact = [anObject retain];

  [allCells release];
  allCells = nil;

  selfResized = YES;
  [self setFrame: NSZeroRect];
  [self layout];
}

- (Contact*) contact
{
  return contact;
}

- (void) setDelegate: (id) anObject
{
  delegate = anObject;
}

- (id) delegate
{
  return delegate;
}

- (void) drawRect: (NSRect) rect
{
  NSEnumerator *e; id cell;
  
  [super drawRect: rect];

  if(!contact) return;
  
  [self lockFocus];

  if(![self isEditable])
    [[NSColor whiteColor] set];
  else
    [[NSColor colorWithDeviceRed: 1.0 green: 1.0 blue: 0.5 alpha: 1.0] set];
  NSRectFill(rect);

  e = [allCells objectEnumerator];
  while((cell = [e nextObject]))
    {
      if(cell != editingCell)
	[cell drawInView: self];
    }

  [self unlockFocus];
}

- (BOOL) isEditable
{
  return editable;
}

- (void) setEditable: (BOOL) yn
{
  if(yn == editable) return;

  editable = yn;
  if(!yn && editingCell)
    {
      [editingCell endEditing: textObject];
      textObject = nil;
      editingCell = nil;
    }
  [self layout];
}

- (void) beginEditingInCell: (id) c
	     becauseOfEvent: (NSEvent*) e
{
  if(editingCell) [self endEditing];

  if([c isKindOf: [ContactDataCell class]])
    {
      editingCell = c;
      NSRect r = [c rect];
      
      NSText *t = [_window fieldEditor: YES forObject: self];
      [t resignFirstResponder];
      [_window makeFirstResponder: _window];

      [c setTextColor: [NSColor blackColor]];
      textObject = [c setUpFieldEditorAttributes: t];
      [textObject setBackgroundColor: [NSColor grayColor]];
      [textObject setDrawsBackground: YES];
      [textObject setString: [c stringValue]];
      
      if([c displaysSubstitute])
	{
	  if([c isKindOf: [ContactDataCell class]])
	    {
	      [c setStringValue: @""];
	      r.size.width = [[c font] widthOfString: @""];
	    }
	}
      
      r.size.width += 2; // make the cursor fit too
      [c editWithFrame: r
	 inView: self
	 editor: textObject
	 delegate: self
	 event: e];
    }
  else
    {
      NSOpenPanel *p = [NSOpenPanel openPanel];
      [p setCanChooseDirectories: NO];
      [p setAllowsMultipleSelection: NO];
      [p runModalForTypes: [NSArray arrayWithObjects: @"tiff", @"jpg", nil]];

      if(![[p filenames] count])
	return;
      NSString *fname = [[p filenames] objectAtIndex: 0];
      if([fname isEqualToString: @""])
	return;
      if(![contact takeImageFromFile: fname])
	NSRunAlertPanel(@"Error",
			[NSString stringWithFormat: @"Couldn't read "
				  @"image from %@", fname],
			@"OK", nil, nil, nil);
    }

  [self setNeedsDisplay: YES];
}

- (void) endEditing
{  
  [editingCell endEditing: textObject];
  textObject = nil;
  editingCell = nil;
  [self layout];
  [self setNeedsDisplay: YES];
}

- (void) runActionForCell: (id) c
	   becauseOfEvent: (NSEvent*) e
{
  [c invokeAction];
}

- (void) mouseDown: (NSEvent*) event
{
  NSPoint p = [self convertPoint: [event locationInWindow]
		    fromView: nil];

  ContactDataCell *c = [self cellContainingPoint: p];
  
  if(!c)
    {
      if(editingCell) [self endEditing];
      [super mouseDown: event];
      return;
    }

  if(editable)
    [self beginEditingInCell: c
	  becauseOfEvent: event];
  else
    [self runActionForCell: c
	  becauseOfEvent: event];
}

/*
  Utility methods
*/

- (float) maxLabelWidthInRect: (NSRect) rect
{
  float w = 0;
  int i;

  for(i=0; i<[labels count]; i++)
    {
      [[labels objectAtIndex: i] calcDrawInfo: rect];
      NSSize s = [[labels objectAtIndex: i] cellSize];
      if(w < s.width) w = s.width;
    }
  return w;
}

- (NSCell*) cellContainingPoint: (NSPoint)aPoint
{
  NSEnumerator *e = [allCells objectEnumerator];
  ContactDataCell *c;

  while((c = [e nextObject]))
    if(NSPointInRect(aPoint, [c rect]))
       return c;
  return nil;
}

- (BOOL) textShouldEndEditing: (NSText*) text
{
  return YES;
}

- (void) textDidEndEditing: (NSNotification*) aNotification
{
  [editingCell endEditing: textObject];
  [editingCell setStringValue: [[[textObject string] copy] autorelease]];
  [editingCell updateContact];
  editingCell = nil;
  textObject = nil;
  [self layout];
  [self setNeedsDisplay: YES];
}

- (void) textDidChange: (NSNotification*) aNotification
{
  NSSize s = [editingCell rect].size;
  s.width = [[editingCell font] widthOfString: [textObject string]] + 2;

  [[textObject superview] setFrameSize: s];
  [textObject setFrameSize: s];
  [textObject setNeedsDisplay: YES];
  [self setNeedsDisplay: YES];
}

/*
  Event methods
*/
- (void) clickedOnURL: (id) cell
{
  NSPasteboard *pb = [NSPasteboard generalPasteboard];
  [pb declareTypes: [NSArray arrayWithObjects: NSStringPboardType, nil]
      owner: self];
  [pb setString: [cell stringValue] forType: NSStringPboardType];

  NSPerformService(@"Open URL", pb);
}

- (void) clickedOnEmail: (id) cell
{
  NSPasteboard *pb = [NSPasteboard generalPasteboard];
  [pb declareTypes: [NSArray arrayWithObjects: NSStringPboardType, nil]
      owner: self];
  [pb setString: [cell stringValue] forType: NSStringPboardType];

  NSPerformService(@"GNUMail/New Mail with recipient", pb);
}

- (void) superviewFrameChanged: (NSNotification*) note
{
  if([self isDescendantOf: [note object]])
    [self layout];
}

@end
