// ContactViewCells.m (this is -*- ObjC -*-)
// 
// \author: Bjrn Giesler <giesler@ira.uka.de>
// 
// 
// 
// $Author: giesler $
// $Locker:  $
// $Revision: 1.3 $
// $Date: 2002/08/19 08:41:28 $

#include "ContactViewCells.h"

@implementation ContactDataCell
- (void) dealloc
{
  [substitute release];
}

- (void) drawWithFrame: (NSRect) frame
		inView: (NSView*) view
{
  _rect = frame;
  if(target && selector && [target respondsToSelector: selector] &&
     ![[self stringValue] isEqualToString: substitute])
    [self setTextColor: [NSColor blueColor]];
  return [super drawWithFrame: frame inView: view];
}

- (void) drawInView: (NSView*) view
{
  return [self drawWithFrame: _rect inView: view];
}

- (void) setRect: (NSRect) aRect
{
  _rect = aRect;
}

- (void) moveDownBy: (float) y
{
  _rect.origin.y -= y;
}

- (NSRect) rect
{
  return _rect;
}

- (void) setRepresentsKey: (NSString*) key
	     valueAtIndex: (int) v
	       parameters: (id) parameters
{
  representedKey = [key retain];
  valueIndex = v;
  representedParameters = [parameters retain];
}

- (void) setSubstitute: (id) subst
{
  substitute = [subst retain];
  if(![self stringValue] ||
     [[self stringValue] isEqualToString: @""])
    {
      [self setTextColor: [NSColor grayColor]];
      [self setStringValue: substitute];
    }
}

- (id) substitute
{
  return substitute;
}

- (BOOL) displaysSubstitute
{
  if([[self stringValue] isEqualToString: substitute])
    return YES;
  return NO;
}

- (id) representedParameters
{
  return representedParameters;
}

- (void) setContact: (Contact*) c
{
  contact = c;
  [self setStringValue: [c valueForKey: representedKey
			   parameter: representedParameters
			   index: valueIndex]];
}

- (void) updateContact
{
  [contact setValue: [self stringValue]
	   forKey: representedKey
	   parameter: representedParameters
	   atIndex: valueIndex];
  [contact writeToFile];
}

- (void) setActionSelector: (SEL) aSelector
{
  selector = aSelector;
}

- (void) setActionTarget: (id) anObject
{
  target = anObject;
}

- (void) invokeAction
{
  if(selector && target &&
     [target respondsToSelector: selector])
    [target performSelector: selector withObject: self];
  else
    NSLog(@"Selector: 0x%x Target: 0x%x responds: %d No go\n",
	  selector, target, [target respondsToSelector: selector]);
}
@end

@implementation ContactImageCell
- (void) dealloc
{
  [substitute release];
}

- (void) drawWithFrame: (NSRect) frame
		inView: (NSView*) view
{
  _rect = frame;
  return [super drawWithFrame: frame inView: view];
}

- (void) drawInView: (NSView*) view
{
  return [super drawWithFrame: _rect inView: view];
}

- (void) setRect: (NSRect) aRect
{
  _rect = aRect;
}

- (void) moveDownBy: (float) y
{
  _rect.origin.y -= y;
}

- (NSRect) rect
{
  return _rect;
}

- (void) setRepresentsKey: (NSString*) key
	     valueAtIndex: (int) v
	       parameters: (id) parameters
{
  representedKey = [key retain];
  valueIndex = v;
  representedParameters = [parameters retain];
}

- (id) representedParameters
{
  return representedParameters;
}

- (void) setContact: (Contact*) c
{
  contact = c;
  if([c image])
    [self setImage: [c image]];
  else
    [self setImage: [NSImage imageNamed: @"UnknownImage"]];
}

- (void) updateContact
{
  [contact setValue: [self stringValue]
	   forKey: representedKey
	   parameter: representedParameters
	   atIndex: valueIndex];
  [contact writeToFile];
}

- (void) setActionSelector: (SEL) aSelector
{
  selector = aSelector;
}

- (void) setSubstitute: (id) subst
{
  substitute = [subst retain];
  if(![self image])
    [self setImage: substitute];
}

- (id) substitute
{
  return substitute;
}

- (BOOL) displaysSubstitute
{
  if([self image] == substitute)
    return YES;
  return NO;
}

- (void) setActionTarget: (id) anObject
{
  target = anObject;
}

- (void) invokeAction
{
  if(selector && target &&
     [target respondsToSelector: selector])
    [target performSelector: selector withObject: self];
}
@end
