// ContactViewDisplayLayouter.m (this is -*- ObjC -*-)
// 
// \author: Bjrn Giesler <giesler@ira.uka.de>
// 
// Layouter for ContactView's display mode
// 
// $Author: giesler $
// $Locker:  $
// $Revision: 1.3 $
// $Date: 2002/08/19 08:41:28 $

#include "ContactViewDisplayLayouter.h"

static float maxLabelWidth;
#define BIG 10000.0

@implementation ContactViewDisplayLayouter
+ (BOOL) _layoutURLsForContact: (Contact*) c
			inRect: (NSRect) rect
			   atY: (float) y
			  newY: (float*) newY
			target: (id) target
{
  NSDictionary *dict = [c dictionary];
  NSArray *addresses = [dict objectForKey: @"url"];
  NSEnumerator *e;
  NSMutableArray *home, *work, *dest;
  id cell;
  NSRect r;
  int i;
  float startX;

  startX = 10.0 + maxLabelWidth + [[self labelFont] widthOfString: @"r"];

  home = [NSMutableArray arrayWithCapacity: [addresses count]];
  work = [NSMutableArray arrayWithCapacity: [addresses count]];
  
  e = [addresses objectEnumerator];
  while((dict = [e nextObject]))
    {
      NSArray *parameters = [dict objectForKey: @"Parameters"];
      NSArray *values = [dict objectForKey: @"Values"];

      dest = home;
      if([parameters containsObject: @"work"])
	dest = work;
      for(i=0; i<[values count]; i++)
	{
	  [dest addObject: [self makeDataFieldForKey: @"url"
				 valueIndex: i
				 parameters: parameters]];
	}
    }

  if([home count])
    {
      cell = [self makeLabel: @"Private URL:"];
      [cell calcDrawInfo: rect];
      r.size.width = maxLabelWidth;
      r.size.height = [cell cellSize].height;
      r.origin.x = 10.0;
      r.origin.y = y - r.size.height;
      [cell setRect: r];

      r.origin.x = startX;
      e = [home objectEnumerator];
      while((cell = [e nextObject]))
	{
	  [cell setContact: c];
	  [cell calcDrawInfo: rect];
	  [cell setActionTarget: target];
	  [cell setActionSelector: @selector(clickedOnURL:)];
	  r.size = [cell cellSize];
	  r.origin.y = y - r.size.height;
	  [cell setRect: r];

	  y -= r.size.height;
	}
    }
      
  if([work count])
    {
      cell = [self makeLabel: @"Business URL:"];
      [cell calcDrawInfo: rect];
      r.size = NSMakeSize(maxLabelWidth, [cell cellSize].height);
      r.origin = NSMakePoint(10.0, y - r.size.height);
      [cell setRect: r];

      r.origin.x = startX;
      e = [work objectEnumerator];
      while((cell = [e nextObject]))
	{
	  [cell setContact: c];
	  [cell calcDrawInfo: rect];
	  [cell setActionTarget: target];
	  [cell setActionSelector: @selector(clickedOnURL:)];
	  r.size = [cell cellSize];
	  r.origin.y = y - r.size.height;
	  [cell setRect: r];

	  y -= r.size.height;
	}
    }

  *newY = y;
  if([home count] || [work count])
    return YES;
  return NO;
}

+ (BOOL) _layoutEmailAddressesForContact: (Contact*) c
				  inRect: (NSRect) rect
				     atY: (float) y
				    newY: (float*) newY
				  target: (id) target
{
  NSDictionary *dict = [c dictionary];
  NSArray *addresses = [dict objectForKey: @"email"];
  NSEnumerator *e;
  NSMutableArray *home, *work, *dest;
  id cell;
  NSRect r;
  int i;
  float startX;

  startX = 10.0 + maxLabelWidth + [[self labelFont] widthOfString: @"r"];

  home = [NSMutableArray arrayWithCapacity: [addresses count]];
  work = [NSMutableArray arrayWithCapacity: [addresses count]];
  
  e = [addresses objectEnumerator];
  while((dict = [e nextObject]))
    {
      NSArray *parameters = [dict objectForKey: @"Parameters"];
      NSArray *values = [dict objectForKey: @"Values"];

      dest = home;
      if([parameters containsObject: @"work"])
	dest = work;
      for(i=0; i<[values count]; i++)
	{
	  [dest addObject: [self makeDataFieldForKey: @"email"
				 valueIndex: i
				 parameters: parameters]];
	}
    }

  if([home count])
    {
      cell = [self makeLabel: @"Private Email:"];
      [cell calcDrawInfo: rect];
      r.size = NSMakeSize(maxLabelWidth, [cell cellSize].height);
      r.origin = NSMakePoint(10.0, y-r.size.height);
      [cell setRect: r];

      r.origin.x = startX;
      e = [home objectEnumerator];
      while((cell = [e nextObject]))
	{
	  [cell setContact: c];
	  [cell calcDrawInfo: rect];
	  [cell setActionTarget: target];
	  [cell setActionSelector: @selector(clickedOnEmail:)];
	  r.size = [cell cellSize];
	  r.origin.y = y - r.size.height;
	  [cell setRect: r];

	  y -= r.size.height;
	}
    }
      
  if([work count])
    {
      cell = [self makeLabel: @"Business Email:"];
      [cell calcDrawInfo: rect];
      r.size = NSMakeSize(maxLabelWidth, [cell cellSize].height);
      r.origin = NSMakePoint(10.0, y-r.size.height);
      [cell setRect: r];

      r.origin.x = startX;
      e = [work objectEnumerator];
      while((cell = [e nextObject]))
	{
	  [cell setContact: c];
	  [cell calcDrawInfo: rect];
	  [cell setActionTarget: target];
	  [cell setActionSelector: @selector(clickedOnEmail:)];
	  r.size = [cell cellSize];
	  r.origin.y = y - r.size.height;
	  [cell setRect: r];

	  y -= r.size.height;
	}
    }

  *newY = y;
  if([home count] || [work count])
    return YES;
  return NO;
}

+ (BOOL) _layoutPhoneNumbersForContact: (Contact*) c
				inRect: (NSRect) rect
				   atY: (float) y
				  newY: (float*) newY
				target: (id) target
{
  NSDictionary *dict = [c dictionary];
  NSArray *addresses = [dict objectForKey: @"tel"];
  NSEnumerator *e;
  NSMutableArray *home, *work, *dest;
  id cell;
  NSRect r;
  int i;
  float startX;
  
  startX = 10.0 + maxLabelWidth + [[self labelFont] widthOfString: @"r"];

  home = [NSMutableArray arrayWithCapacity: [addresses count]];
  work = [NSMutableArray arrayWithCapacity: [addresses count]];
  
  e = [addresses objectEnumerator];
  while((dict = [e nextObject]))
    {
      NSArray *parameters = [dict objectForKey: @"Parameters"];
      NSArray *values = [dict objectForKey: @"Values"];

      dest = home;
      if([parameters containsObject: @"work"])
	dest = work;
      for(i=0; i<[values count]; i++)
	{
	  [dest addObject: [self makeDataFieldForKey: @"tel"
				 valueIndex: i
				 parameters: parameters]];
	}
    }

  if([home count])
    {
      cell = [self makeLabel: @"Private Phone:"];
      [cell calcDrawInfo: rect];
      r.size = NSMakeSize(maxLabelWidth, [cell cellSize].height);
      r.origin = NSMakePoint(10.0, y-r.size.height);
      [cell setRect: r];

      e = [home objectEnumerator];
      while((cell = [e nextObject]))
	{
	  y -= r.size.height;

	  ContactImageCell *ic = [[ContactImageCell alloc] init];
	  [ic setRect: NSMakeRect(startX, y, 16, 16)];
	  if([[cell representedParameters] containsObject: @"cell"])
	    [ic setImage: [NSImage imageNamed: @"CellPhone"]];
	  else
	    [ic setImage: [NSImage imageNamed: @"Phone"]];
	  [[self allCells] addObject: ic];

	  [cell setContact: c];
	  [cell setActionTarget: target];
	  [cell setActionSelector: @selector(clickedOnPhone:)];
	  [cell calcDrawInfo: rect];
	  r.size = [cell cellSize];
	  r.origin = NSMakePoint(startX + 20, y);
	  [cell setRect: r];
	}
    }
      
  if([work count])
    {
      cell = [self makeLabel: @"Business Phone:"];
      [cell calcDrawInfo: rect];
      r.size = NSMakeSize(maxLabelWidth, [cell cellSize].height);
      r.origin = NSMakePoint(10.0, y-r.size.height);
      [cell setRect: r];

      e = [work objectEnumerator];
      while((cell = [e nextObject]))
	{
	  y -= r.size.height;
	  
	  ContactImageCell *ic = [[ContactImageCell alloc] init];
	  [ic setRect: NSMakeRect(startX, y, 16, 16)];
	  if([[cell representedParameters] containsObject: @"cell"])
	    [ic setImage: [NSImage imageNamed: @"CellPhone"]];
	  else
	    [ic setImage: [NSImage imageNamed: @"Phone"]];
	  [[self allCells] addObject: ic];

	  [cell setContact: c];
	  [cell setActionTarget: target];
	  [cell setActionSelector: @selector(clickedOnPhone:)];
	  [cell calcDrawInfo: rect];
	  r.size = [cell cellSize];
	  r.origin = NSMakePoint(startX + 20, y);
	  [cell setRect: r];
	}
    }

  *newY = y;
  if([home count] || [work count])
    return YES;
  return NO;
}

+ (void) _layoutAddress: (NSDictionary*) dict
	     forContact: (Contact*) c
		 inRect: (NSRect) rect
		    atY: (float) y
		   newY: (float*) newY
		 target: (id) target
{
  NSArray *layout = [self layoutForAddress: dict];
  NSArray *addr = [dict objectForKey: @"Values"];
  NSArray *parameters = [dict objectForKey: @"Parameters"];

  NSArray *row; NSString *entry;
  NSEnumerator *rowEnum, *e;
  id cell; float x, startX, rowHeight;
  NSRect r;
  BOOL rowEmpty;
  int index;

  if([[dict objectForKey: @"Parameters"] containsObject: @"work"])
    cell = [self makeLabel: @"Business Address:"];
  else
    cell = [self makeLabel: @"Private Address:"];
  [cell calcDrawInfo: rect];
  r.size = NSMakeSize(maxLabelWidth, [cell cellSize].height);
  r.origin = NSMakePoint(10.0, y-r.size.height);
  [cell setRect: r];

  startX = r.origin.x + r.size.width + [[self labelFont] widthOfString: @"r"];

  rowEnum = [layout objectEnumerator];
  while((row = [rowEnum nextObject]))
    {
      x = startX;
      rowHeight = 0;
      rowEmpty = YES;

      // first, check whether the row will be empty
      e = [row objectEnumerator];
      while((entry = [e nextObject]))
	{
	  if([self addressStringConstant: entry]) continue;
	  index = [Contact addressIndexForAddressField: entry];
	  if(index >= [addr count])
	    continue;
	  entry = [addr objectAtIndex: index];
	  if(!entry || [entry isEqualToString: @""])
	    continue;
	  rowEmpty = NO;
	  break;
	}

      if(rowEmpty)
	continue;

      // still here? make row
      e = [row objectEnumerator];
      while((entry = [e nextObject]))
	{
	  // string constant?
	  if([self addressStringConstant: entry])
	    {
	      cell = [self makeDataFieldForKey: nil
			   valueIndex: 0
			   parameters: nil];
	      [cell setStringValue: [self addressStringConstant: entry]];
	      [cell calcDrawInfo: rect];
	      r.size = [cell cellSize];
	      r.origin = NSMakePoint(x, y-r.size.height);
	      [cell setRect: r];

	      rowHeight = MAX(rowHeight, r.size.height);
	      x += r.size.width + [[self entryFont] widthOfString: @"r"];

	      continue;
	    }

	  // get address book entry for this
	  index = [Contact addressIndexForAddressField: entry];
	  if(index >= [addr count])
	    {
	      NSLog(@"Trying to access object at %d, which doesn't exist\n",
		    index);
	      continue;
	    }
	  entry = [addr objectAtIndex: index];
	  if(!entry || [entry isEqualToString: @""])
	    continue;

	  cell = [self makeDataFieldForKey: @"adr"
		       valueIndex: index
		       parameters: parameters];
	  [cell setContact: c];
	  [cell calcDrawInfo: rect];
	  r.size = [cell cellSize];
	  r.origin = NSMakePoint(x, y-r.size.height);
	  [cell setRect: r];

	  rowHeight = MAX(rowHeight, r.size.height);
	  x += r.size.width + [[self entryFont] widthOfString: @"r"];
	}
      y -= rowHeight;
    }

  *newY = y;
}

+ (BOOL) _layoutAddressesForContact: (Contact*) c
			     inRect: (NSRect) rect
				atY: (float) y
			       newY: (float*) newY
			     target: (id) target
{
  NSDictionary *dict = [c dictionary];
  NSArray *addresses = [dict objectForKey: @"adr"];
  NSEnumerator *e;
  BOOL didSomething;

  e = [addresses objectEnumerator];
  while((dict = [e nextObject]))
    {
      [self _layoutAddress: dict
	    forContact: c
	    inRect: rect
	    atY: y
	    newY: newY
	    target: target];
      y = *newY;
      didSomething = YES;
    }
  return didSomething;
}

+ (NSArray*) doLayoutForContact: (Contact*) c
		      needsSize: (NSSize*) s
			 target: (id) target
{
  ContactDataCell *dc; ContactImageCell *ic; id cell;
  NSRect rect, r;
  NSEnumerator *e;
  
  rect = NSMakeRect(0, 0, BIG, BIG);
  
  float lastY;

  [self init];
  
  maxLabelWidth = 
    [self maxLabelWidth: [NSArray arrayWithObjects: @"Private URL:",
				  @"Business URL:", @"Private Email:",
				  @"Business Email:", @"Private Phone:",
				  @"Business Phone:", @"Private Address:",
				  @"Business Address:", nil]
	  forFont: [self labelFont]];

  /*
   * Picture, Name, Company, Title. These are always there.
   */

  ic = [[ContactImageCell alloc] init];
  [ic setRepresentsKey: @"photo" valueAtIndex: 0 parameters: nil];
  [ic setImageScaling: NSScaleProportionally];
  [ic setContact: c];
  [ic setRect: NSMakeRect(rect.origin.x + 10.0,
			  rect.origin.y + rect.size.height - 70.0,
			  64.0, 50.0)];
  [[self allCells] addObject: ic];

  dc = [self makeDataFieldForKey: @"n" valueIndex: 1]; // first name
  [dc setFont: [self nameFont]];
  [dc setContact: c];
  [dc calcDrawInfo: rect];
  r.origin.x = 84.0;
  r.size = [dc cellSize];
  r.origin.y = rect.origin.y + rect.size.height - r.size.height - 10.0;
  [dc setRect: r];
  s->width = MAX(s->width, r.origin.x + r.size.width);

  dc = [self makeDataFieldForKey: @"n" valueIndex: 0]; // last name
  [dc setFont: [self nameFont]];
  [dc setContact: c];
  [dc calcDrawInfo: rect];
  r.origin.x += r.size.width + [[self nameFont] widthOfString: @"r"];
  r.size = [dc cellSize];
  [dc setRect: r];
  s->width = MAX(s->width, r.origin.x + r.size.width);

  dc = [self makeDataFieldForKey: @"org"]; // company
  [dc setContact: c];
  [dc calcDrawInfo: rect];
  r.origin.x = 84.0;
  r.size = [dc cellSize];
  r.origin.y -= r.size.height;
  [dc setRect: r];
  s->width = MAX(s->width, r.origin.x + r.size.width);

  dc = [self makeDataFieldForKey: @"title"]; // title
  [dc setContact: c];
  [dc calcDrawInfo: rect];
  r.origin.x = 84.0;
  r.size = [dc cellSize];
  r.origin.y -= r.size.height;
  [dc setRect: r];
  s->width = MAX(s->width, r.origin.x + r.size.width);

  /*
    Email, Phone Numbers, Addresses. These are optional.
  */

  lastY = rect.origin.y + rect.size.height - 80.0;
  if([self _layoutURLsForContact: c
	   inRect: rect
	   atY: lastY
	   newY: &lastY
	   target: target])
    lastY -= 10.0;
  if([self _layoutEmailAddressesForContact: c
	   inRect: rect
	   atY: lastY
	   newY: &lastY
	   target: target])
    lastY -= 10.0;
  if([self _layoutPhoneNumbersForContact: c
	   inRect: rect
	   atY: lastY
	   newY: &lastY
	   target: target])
    lastY -= 10.0;
  [self _layoutAddressesForContact: c
	inRect: rect
	atY: lastY
	newY: &lastY
	target: target];

  /*
    Check how much space we really need.
  */
  r.origin.y = lastY;
  s->height = BIG - r.origin.y;
  e = [[self allCells] objectEnumerator];
  while((cell = [e nextObject]))
    {
      [cell moveDownBy: BIG - s->height - 10.0];
      s->width = MAX(s->width, [cell rect].origin.x+[cell rect].size.width);
    }
  s->width += 10.0;
  
  return [self allCells];
}

@end
