// ContactViewEditLayouter.m (this is -*- ObjC -*-)
// 
// \author: Bjrn Giesler <giesler@ira.uka.de>
// 
// 
// 
// $Author: giesler $
// $Locker:  $
// $Revision: 1.3 $
// $Date: 2002/08/19 08:41:28 $

#include "ContactViewEditLayouter.h"

static float maxLabelWidth;
#define BIG 10000.0

@implementation ContactViewEditLayouter
+ (BOOL) _layoutURLsForContact: (Contact*) c
			inRect: (NSRect) rect
			   atY: (float) y
			  newY: (float*) newY
			target: (id) target
{
  NSDictionary *dict = [c dictionary];
  NSArray *addresses = [dict objectForKey: @"url"];
  NSEnumerator *e;
  NSMutableArray *home, *work, *dest;
  id cell;
  NSRect r;
  int i;
  float startX;

  startX = 10.0 + maxLabelWidth + [[self labelFont] widthOfString: @"r"];

  home = [NSMutableArray arrayWithCapacity: [addresses count]];
  work = [NSMutableArray arrayWithCapacity: [addresses count]];
  
  e = [addresses objectEnumerator];
  while((dict = [e nextObject]))
    {
      NSArray *parameters = [dict objectForKey: @"Parameters"];
      NSArray *values = [dict objectForKey: @"Values"];

      dest = home;
      if([parameters containsObject: @"work"])
	dest = work;
      for(i=0; i<[values count]; i++)
	{
	  [dest addObject: [self makeDataFieldForKey: @"url"
				 valueIndex: i
				 parameters: parameters]];
	}
    }

  if(![home count])
    [home addObject: [self makeDataFieldForKey: @"url"
			   valueIndex: 0
			   parameters: @"home"]];
  if(![work count])
    [work addObject: [self makeDataFieldForKey: @"url"
			   valueIndex: 0
			   parameters: @"work"]];
  
  cell = [self makeLabel: @"Private URL:"];
  [cell calcDrawInfo: rect];
  r.size = NSMakeSize(maxLabelWidth, [cell cellSize].height);
  r.origin = NSMakePoint(10.0, y-r.size.height);
  [cell setRect: r];
  
  r.origin.x = startX;
  e = [home objectEnumerator];
  while((cell = [e nextObject]))
    {
      [cell setContact: c];
      [cell calcDrawInfo: rect];
      [cell setActionTarget: target];
      [cell setActionSelector: @selector(clickedOnEmail:)];
      [cell setSubstitute: @"[http://www.host.com]"];
      r.size = [cell cellSize];
      r.origin.y = y - r.size.height;
      [cell setRect: r];
      
      y -= r.size.height;
    }
      
  cell = [self makeLabel: @"Business URL:"];
  [cell calcDrawInfo: rect];
  r.size = NSMakeSize(maxLabelWidth, [cell cellSize].height);
  r.origin = NSMakePoint(10.0, y-r.size.height);
  [cell setRect: r];
  
  r.origin.x = startX;
  e = [work objectEnumerator];
  while((cell = [e nextObject]))
    {
      [cell setContact: c];
      [cell calcDrawInfo: rect];
      [cell setActionTarget: target];
      [cell setActionSelector: @selector(clickedOnEmail:)];
      [cell setSubstitute: @"[http://www.host.com]"];
      r.size = [cell cellSize];
      r.origin.y = y - r.size.height;
      [cell setRect: r];
      
      y -= r.size.height;
    }

  *newY = y;
  if([home count] || [work count])
    return YES;
  return NO;
}
+ (BOOL) _layoutEmailAddressesForContact: (Contact*) c
				  inRect: (NSRect) rect
				     atY: (float) y
				    newY: (float*) newY
				  target: (id) target
{
  NSDictionary *dict = [c dictionary];
  NSArray *addresses = [dict objectForKey: @"email"];
  NSEnumerator *e;
  NSMutableArray *home, *work, *dest;
  id cell;
  NSRect r;
  int i;
  float startX;

  startX = 10.0 + maxLabelWidth + [[self labelFont] widthOfString: @"r"];

  home = [NSMutableArray arrayWithCapacity: [addresses count]];
  work = [NSMutableArray arrayWithCapacity: [addresses count]];
  
  e = [addresses objectEnumerator];
  while((dict = [e nextObject]))
    {
      NSArray *parameters = [dict objectForKey: @"Parameters"];
      NSArray *values = [dict objectForKey: @"Values"];

      dest = home;
      if([parameters containsObject: @"work"])
	dest = work;
      for(i=0; i<[values count]; i++)
	{
	  [dest addObject: [self makeDataFieldForKey: @"email"
				 valueIndex: i
				 parameters: parameters]];
	}
    }

  if(![home count])
    [home addObject: [self makeDataFieldForKey: @"email"
			   valueIndex: 0
			   parameters: @"home"]];
  if(![work count])
    [work addObject: [self makeDataFieldForKey: @"email"
			   valueIndex: 0
			   parameters: @"work"]];
  
  cell = [self makeLabel: @"Private Email:"];
  [cell calcDrawInfo: rect];
  r.size = NSMakeSize(maxLabelWidth, [cell cellSize].height);
  r.origin = NSMakePoint(10.0, y-r.size.height);
  [cell setRect: r];
  
  r.origin.x = startX;
  e = [home objectEnumerator];
  while((cell = [e nextObject]))
    {
      [cell setContact: c];
      [cell calcDrawInfo: rect];
      [cell setActionTarget: target];
      [cell setActionSelector: @selector(clickedOnEmail:)];
      [cell setSubstitute: @"[name@host.com]"];
      r.size = [cell cellSize];
      r.origin.y = y - r.size.height;
      [cell setRect: r];
      
      y -= r.size.height;
    }
      
  cell = [self makeLabel: @"Business Email:"];
  [cell calcDrawInfo: rect];
  r.size = NSMakeSize(maxLabelWidth, [cell cellSize].height);
  r.origin = NSMakePoint(10.0, y-r.size.height);
  [cell setRect: r];
  
  r.origin.x = startX;
  e = [work objectEnumerator];
  while((cell = [e nextObject]))
    {
      [cell setContact: c];
      [cell calcDrawInfo: rect];
      [cell setActionTarget: target];
      [cell setActionSelector: @selector(clickedOnEmail:)];
      [cell setSubstitute: @"[name@host.com]"];
      r.size = [cell cellSize];
      r.origin.y = y - r.size.height;
      [cell setRect: r];
      
      y -= r.size.height;
    }

  *newY = y;
  if([home count] || [work count])
    return YES;
  return NO;
}

+ (BOOL) _layoutPhoneNumbersForContact: (Contact*) c
				inRect: (NSRect) rect
				   atY: (float) y
				  newY: (float*) newY
				target: (id) target
{
  NSDictionary *dict = [c dictionary];
  NSArray *addresses = [dict objectForKey: @"tel"];
  NSEnumerator *e;
  NSMutableArray *home, *work, *dest;
  id cell;
  NSRect r;
  int i;
  float startX;

  startX = 10.0 + maxLabelWidth + [[self labelFont] widthOfString: @"r"];

  home = [NSMutableArray arrayWithCapacity: [addresses count]];
  work = [NSMutableArray arrayWithCapacity: [addresses count]];
  
  e = [addresses objectEnumerator];
  while((dict = [e nextObject]))
    {
      NSArray *parameters = [dict objectForKey: @"Parameters"];
      NSArray *values = [dict objectForKey: @"Values"];

      dest = home;
      if([parameters containsObject: @"work"])
	dest = work;
      for(i=0; i<[values count]; i++)
	{
	  [dest addObject: [self makeDataFieldForKey: @"tel"
				 valueIndex: i
				 parameters: parameters]];
	}
    }

  if(![home count])
    [home addObject: [self makeDataFieldForKey: @"tel"
			   valueIndex: 0
			   parameters: @"home"]];
  if(![work count])
    [work addObject: [self makeDataFieldForKey: @"tel"
			   valueIndex: 0
			   parameters: @"work"]];
  
  cell = [self makeLabel: @"Private Phone:"];
  [cell calcDrawInfo: rect];
  r.size = NSMakeSize(maxLabelWidth, [cell cellSize].height);
  r.origin = NSMakePoint(10.0, y-r.size.height);
  [cell setRect: r];
  
  r.origin.x = startX;
  e = [home objectEnumerator];
  while((cell = [e nextObject]))
    {
      [cell setContact: c];
      [cell calcDrawInfo: rect];
      [cell setActionTarget: target];
      [cell setActionSelector: @selector(clickedOnEmail:)];
      [cell setSubstitute: @"[+00-000-000-0000]"];
      r.size = [cell cellSize];
      r.origin.y = y - r.size.height;
      [cell setRect: r];
      
      y -= r.size.height;
    }
      
  cell = [self makeLabel: @"Business Phone:"];
  [cell calcDrawInfo: rect];
  r.size = NSMakeSize(maxLabelWidth, [cell cellSize].height);
  r.origin = NSMakePoint(10.0, y-r.size.height);
  [cell setRect: r];
  
  r.origin.x = startX;
  e = [work objectEnumerator];
  while((cell = [e nextObject]))
    {
      [cell setContact: c];
      [cell calcDrawInfo: rect];
      [cell setActionTarget: target];
      [cell setActionSelector: @selector(clickedOnEmail:)];
      [cell setSubstitute: @"[+00-000-000-0000]"];
      r.size = [cell cellSize];
      r.origin.y = y - r.size.height;
      [cell setRect: r];
      
      y -= r.size.height;
    }

  *newY = y;
  if([home count] || [work count])
    return YES;
  return NO;
}

+ (NSArray*) doLayoutForContact: (Contact*) c
		      needsSize: (NSSize*) s
			 target: (id) target
{
  ContactImageCell *ic; ContactDataCell *dc; id cell;
  NSRect rect, r; NSEnumerator *e;
  float lastY;

  rect = NSMakeRect(0, 0, BIG, BIG);
  
  [self init];

  // allow for change image at end of labels
  maxLabelWidth = 
    [self maxLabelWidth: [NSArray arrayWithObjects: @"Private URL:",
				  @"Business URL:", @"Private Email:",
				  @"Business Email:", @"Private Phone:",
				  @"Business Phone:", @"Private Address:",
				  @"Business Address:", nil]
	  forFont: [self labelFont]] + 9;


  /*
   * Picture, Name, Company, Title. These are always there.
   */

  ic = [[ContactImageCell alloc] init];
  [ic setRepresentsKey: @"photo" valueAtIndex: 0 parameters: nil];
  [ic setImageScaling: NSScaleProportionally];
  [ic setContact: c];
  [ic setRect: NSMakeRect(rect.origin.x + 10.0,
			  rect.origin.y + rect.size.height - 70.0,
			  64.0, 50.0)];
  [[self allCells] addObject: ic];

  dc = [self makeDataFieldForKey: @"n" valueIndex: 1]; // first name
  [dc setFont: [self nameFont]];
  [dc setContact: c];
  [dc setSubstitute: @"[First name]"];
  [dc calcDrawInfo: rect];
  r.origin.x = 84.0;
  r.size = [dc cellSize];
  r.origin.y = rect.origin.y + rect.size.height - r.size.height - 10.0;
  [dc setRect: r];
  s->width = MAX(s->width, r.origin.x + r.size.width);

  dc = [self makeDataFieldForKey: @"n" valueIndex: 0]; // last name
  [dc setFont: [self nameFont]];
  [dc setContact: c];
  [dc setSubstitute: @"[Last name]"];
  [dc calcDrawInfo: rect];
  r.origin.x += r.size.width + [[self nameFont] widthOfString: @"r"];
  r.size = [dc cellSize];
  [dc setRect: r];
  s->width = MAX(s->width, r.origin.x + r.size.width);

  dc = [self makeDataFieldForKey: @"org"]; // company
  [dc setContact: c];
  [dc setSubstitute: @"[Organization]"];
  [dc calcDrawInfo: rect];
  r.origin.x = 84.0;
  r.size = [dc cellSize];
  r.origin.y -= r.size.height;
  [dc setRect: r];
  s->width = MAX(s->width, r.origin.x + r.size.width);

  dc = [self makeDataFieldForKey: @"title"]; // title
  [dc setContact: c];
  [dc setSubstitute: @"[Title]"];
  [dc calcDrawInfo: rect];
  r.origin.x = 84.0;
  r.size = [dc cellSize];
  r.origin.y -= r.size.height;
  [dc setRect: r];
  s->width = MAX(s->width, r.origin.x + r.size.width);

  lastY = rect.origin.y + rect.size.height - 80.0;

  if([self _layoutURLsForContact: c
	   inRect: rect
	   atY: lastY
	   newY: &lastY
	   target: target])
    lastY -= 10.0;
  if([self _layoutEmailAddressesForContact: c
	   inRect: rect
	   atY: lastY
	   newY: &lastY
	   target: target])
    lastY -= 10.0;
  if([self _layoutPhoneNumbersForContact: c
	   inRect: rect
	   atY: lastY
	   newY: &lastY
	   target: target])
    lastY -= 10.0;

  /*
    Check how much space we really need.
  */
  r.origin.y = lastY;
  s->height = BIG - r.origin.y;
  e = [[self allCells] objectEnumerator];
  while((cell = [e nextObject]))
    {
      [cell moveDownBy: BIG - s->height - 10.0];
      s->width = MAX(s->width, [cell rect].origin.x+[cell rect].size.width);
    }
  s->width += 10.0;
  
  return [self allCells];
}
@end
