// ContactViewLayouter.m (this is -*- ObjC -*-)
// 
// \author: Bjrn Giesler <giesler@ira.uka.de>
// 
// Superclass for ContactView{Display,Edit}Layouter
// 
// $Author: giesler $
// $Locker:  $
// $Revision: 1.3 $
// $Date: 2002/08/19 08:41:28 $

#include "ContactViewLayouter.h"

static NSMutableArray *allCells;

@implementation ContactViewLayouter
+ (void) init
{
  allCells = [NSMutableArray arrayWithCapacity: 10];
}

+ (NSMutableArray*) allCells
{
  return allCells;
}

+ (NSArray*) defaultAddressLayout
{
  NSString *p = [[NSBundle bundleForClass: self]
		  pathForResource: @"German"
		  ofType:@"adresslayout"];
  if(p) return [NSArray arrayWithContentsOfFile: p];

  else // german layout is the compiled-in default
    return [@"((\"$Postfach\", POBox),"
	     @"(Street),"
	     @"(ExtendedAddress),"
	     @"(ZipCode, City),"
	     @"(State),"
	     @"(Country))" propertyList];
}

+ (NSArray*) addressLayoutForName: (NSString*) name
{
  // try to open
  NSString *p = [[NSBundle bundleForClass: self]
		  pathForResource: name
		  ofType:@"adresslayout"];
  NSLog(@"It's at %@\n", p);
  if(!p)
    return [self defaultAddressLayout];
  return
    [NSArray arrayWithContentsOfFile: p];
}

+ (NSArray*) layoutForAddress: (NSDictionary*) dict
{
  NSEnumerator *e = [[dict objectForKey: @"Parameters"] objectEnumerator];
  NSString *str, *layoutName;

  layoutName = nil;
  while((str = [e nextObject]))
    {
      NSArray *arr = [str componentsSeparatedByString: @"="];
      if([arr count] != 2) continue;
      if([[arr objectAtIndex: 0] isEqualToString: @"X-NationalAddressLayout"])
	{
	  layoutName = [arr objectAtIndex: 1];
	  break;
	}
    }

  if(!layoutName)
    return [self defaultAddressLayout];
  return [self addressLayoutForName: layoutName];
}

+ (NSString*) addressStringConstant: (NSString*) str
{
  if([[str substringWithRange: NSMakeRange(0, 1)] isEqualToString: @"$"])
    return [str substringFromIndex: 1];
  return nil;
}

+ (NSFont*) nameFont
{
  return [NSFont boldSystemFontOfSize: 18];
}

+ (NSFont*) labelFont
{
  return [NSFont boldSystemFontOfSize: 12];
}

+ (NSFont*) entryFont
{
  return [NSFont systemFontOfSize: 12];
}

+ (NSArray*) doLayoutForContact: (Contact*) c
		      needsSize: (NSSize*) s
			 target: (id) target
{
  [self subclassResponsibility: _cmd];
  return nil;
}

+ (NSTextFieldCell *) makeLabel: (NSString*) title
{
  NSTextFieldCell *retval = [self makeDataFieldForKey: nil];
  [retval setFont: [self labelFont]];
  [retval setAlignment: NSRightTextAlignment];
  [retval setStringValue: title];
  [retval autorelease];

  return retval;
}

+ (NSTextFieldCell *) makeDataFieldForKey: (NSString*) key
{
  return [self makeDataFieldForKey: key valueIndex: 0 parameters: nil];
}

+ (NSTextFieldCell *) makeDataFieldForKey: (NSString*) key
			       valueIndex: (int) v
{
  return [self makeDataFieldForKey: key valueIndex: v parameters: nil];
}

+ (NSTextFieldCell *) makeDataFieldForKey: (NSString*) key
			       valueIndex: (int) v
			       parameters: (id) arr
{
  ContactDataCell *cell = [[ContactDataCell alloc] init];
  [cell setFont: [self entryFont]];
  [cell setAlignment: NSLeftTextAlignment];
  [cell setEditable: YES];
  [cell setRepresentsKey: key valueAtIndex: v parameters: arr];

  [allCells addObject: cell];
  // [cell autorelease];

  return cell;
}

+ (float) maxLabelWidth: (NSArray*) arr
		forFont: (NSFont*) font
{
  float w = 0;
  NSEnumerator *e;
  NSString *s;

  e = [arr objectEnumerator];
  while((s = [e nextObject]))
    if([font widthOfString: s] > w)
      w = [font widthOfString: s];

  // calculation doesn't work right (often cuts of ":");
  // make this slightly larger
  return w + [font widthOfString: @"r"];
}
@end
