#include <AppKit/AppKit.h>
#include "Controller.h"
#include "Contact.h"
#include "VCFParser.h"
#include "DragDropMatrix.h"

static NSString* abRootPath;

BOOL createDirectoryHierarchy(NSString* abRootPath)
{
  int i;
  NSString *currentPath;
  NSFileManager *fm = [NSFileManager defaultManager];
  NSArray *arr = [abRootPath pathComponents];

  currentPath = [arr objectAtIndex: 0];
  for(i=1; i<[arr count]; i++)
    {
      currentPath = [currentPath
		      stringByAppendingPathComponent: [arr objectAtIndex: i]];

      BOOL dir, result;
      result = [fm fileExistsAtPath: currentPath isDirectory: &dir];
      if((result == YES) && (dir == NO))
	return NO;

      if(result == NO)
	result = [fm createDirectoryAtPath: currentPath attributes: nil];

      if(result == NO)
	return NO;
    }

  return YES;
}

@implementation Controller
+ (NSString*) databaseRoot
{
  return abRootPath;
}

- (void) applicationDidFinishLaunching: (NSNotification*) note
{
  [NSApp registerServicesMenuSendTypes: [NSArray arrayWithObjects:
						   NSStringPboardType,
						 nil]
	 returnTypes: nil];
  
  servicesMenu = [[[NSApp mainMenu]
		    itemWithTitle: @"Services"]
		   submenu];
  [NSApp setServicesMenu: servicesMenu];

  [groupsBrowser selectRow: 0 inColumn: 0];
  [groupsBrowser selectRow: 0 inColumn: 1];
  NSString *group = [[groupsBrowser selectedCellInColumn: 0] stringValue];
  NSArray *contacts = [self contactsInGroup: group];
  if([contacts count])
    {
      [contactView setContact: [contacts objectAtIndex: 0]];
      [clipView
	scrollToPoint: NSMakePoint(0, [contactView frame].size.height)];
    }
}

- (void) awakeFromNib
{
  abRootPath = NSHomeDirectoryForUser(NSUserName());
  abRootPath = [abRootPath stringByAppendingPathComponent: @"GNUstep"];
  abRootPath = [abRootPath stringByAppendingPathComponent: @"Library"];
  abRootPath = [abRootPath stringByAppendingPathComponent: @"Contacts"];
  [abRootPath retain];

  fm = [NSFileManager defaultManager];

  BOOL ok;
  if(![fm fileExistsAtPath: abRootPath isDirectory: &ok])
    ok = NO;

  if(!ok)
    {
      int result =
	NSRunAlertPanel(@"Database directory not found",
			[NSString stringWithFormat:
				    @"The directory %@\n"
				  @"was not found. I cannot continue without it.\n"
				  @"Do you want me to create it for you?",
				  abRootPath],
			@"Yes", @"No", nil, nil);
      if(!result)
	[NSApp terminate: self];

      result = createDirectoryHierarchy(abRootPath);
      if(!result)
	{
	  NSRunAlertPanel(@"Error creating database",
			  [NSString stringWithFormat:
				      @"I could not create the directory %@\n"
				    @"or one of its parent directories. Terminating.", abRootPath],
			  @"OK", nil, nil, nil);
	  [NSApp terminate: self];
	}
    }

  [groupsBrowser setAllowsEmptySelection: NO];
  [groupsBrowser setAllowsMultipleSelection: NO];
  [groupsBrowser setMaxVisibleColumns: 2];
  [groupsBrowser setDelegate: self];
  [groupsBrowser setTitle: @"Group" ofColumn: 0];
  [groupsBrowser setTitle: @"Name" ofColumn: 1];
  [groupsBrowser setMatrixClass: [DragDropMatrix class]];
  [groupsBrowser setTarget: self];
  [groupsBrowser setAction: @selector(browserAction:)];

  clipView = [[NSClipView alloc] initWithFrame: [scrollView frame]];
  [clipView setAutoresizesSubviews: YES];
  contactView = [[ContactView alloc] initWithFrame: NSZeroRect];
  [contactView setFillsSuperview: YES];
  [clipView setDocumentView: contactView];
  [scrollView setContentView: clipView];
  [scrollView setHasVerticalScroller: YES];
  [scrollView setHasHorizontalScroller: YES];
  [scrollView setBorderType: NSBezelBorder];
}

- (NSArray*) groupNames
{
  NSMutableArray *retval = [NSMutableArray arrayWithCapacity: 5];
  NSArray *arr = [fm directoryContentsAtPath: abRootPath];
  int i;
  
  for(i=0; i<[arr count]; i++)
    {
      NSString *path =
	[abRootPath stringByAppendingPathComponent: [arr objectAtIndex: i]];
      BOOL dir;
      [fm fileExistsAtPath: path isDirectory: &dir];
      if(dir)
	[retval addObject: [arr objectAtIndex: i]];
    }
  return [retval sortedArrayUsingSelector: @selector(compare:)];
}

- (BOOL) renameGroup: (NSString*) oldName
		  to: (NSString*) newName
{
  NSString *oldPath = [abRootPath stringByAppendingPathComponent: oldName];
  NSString *newPath = [abRootPath stringByAppendingPathComponent: newName];
  return [fm movePath: oldPath toPath: newPath handler: nil];
}

- (NSArray*) contactsInGroup: (NSString*) group
{
  NSMutableArray *retval = [NSMutableArray arrayWithCapacity: 5];
  NSArray *arr;
  NSString *path = abRootPath;
  int i;

  if((group == nil || [group isEqualToString: @"All"]) && allCache)
    return allCache;
  
  if(![group isEqualToString: @"All"])
    path = [abRootPath stringByAppendingPathComponent: group];

  arr = [fm directoryContentsAtPath: path];
  for(i=0; i<[arr count]; i++)
    {
      NSString *path =
	[abRootPath stringByAppendingPathComponent: [arr objectAtIndex: i]];
      BOOL dir;
      [fm fileExistsAtPath: path isDirectory: &dir];
      if(!dir)
	[retval addObject: [[Contact alloc]
			     initWithContentsOfFile: path]];
    }

  arr = [retval sortedArrayUsingSelector: @selector(screenNameCompare:)];
  if([group isEqualToString: @"All"])
    allCache = [arr retain];
  return arr;
}

- (void) linkContactAtRow: (int) contactRow
	     toGroupAtRow: (int) groupRow
{
  NSString *group = [[groupsBrowser loadedCellAtRow: groupRow
				    column: 0]
		      stringValue];
  NSString *selectedGroup = [[groupsBrowser selectedCellInColumn: 0]
			      stringValue];
  NSString *fname = [[[self contactsInGroup: selectedGroup]
		       objectAtIndex: contactRow]
		      fileName];
  NSString *srcfullfname = [@".." stringByAppendingPathComponent: fname];
  NSString *dstfullfname = [[abRootPath stringByAppendingPathComponent: group]
			     stringByAppendingPathComponent: fname];

  if(![fm createSymbolicLinkAtPath: dstfullfname
	  pathContent: srcfullfname])
    NSRunAlertPanel(@"Link failed",
		    [NSString stringWithFormat:
				@"Could not link from %@\n"

			      @"to %@", srcfullfname, dstfullfname],
		    @"OK", nil, nil, nil);
}

- (void) displayContact: (Contact*) contact
{
  [contactView setContact: contact];
  [clipView scrollToPoint: NSMakePoint(0, [contactView frame].size.height)];
}

- (int) nextValidContactId
{
  int i=1;
  while(1)
    {
      NSString *fname = [NSString stringWithFormat: @"%d", i];
      NSString *path = [abRootPath stringByAppendingPathComponent: fname];
      if(![fm fileExistsAtPath: path])
	return i;
      i++;
    }
}

- (void) deleteContact: (Contact *) contact
	     fromGroup: (NSString *) group
    askForConfirmation: (BOOL) confirm
{
  NSString *path;
  if(group)
    path = [abRootPath stringByAppendingPathComponent: group];
  else
    path = abRootPath;
  
  path = [path stringByAppendingPathComponent: [contact fileName]];

  if(![fm fileExistsAtPath: path])
    return;

  if(confirm)
    {
      if(group)
	{
	  if(!NSRunAlertPanel(@"Delete contact from group?",
			      [NSString stringWithFormat:
					  @"Do you really want to "
					@"delete '%@' from the group '%@'?\n"
					@"(The contact will remain in 'All'.)",
					[contact screenName], group],
			      @"Yes", @"No", nil, nil))
	    return;
	}
      else
	{
	  if(!NSRunAlertPanel(@"Delete contact?",
			      [NSString stringWithFormat:
					  @"Do you really want to "
					@"delete '%@' from the database?",
					[contact screenName]],
			      @"Yes", @"No", nil, nil))
	    return;
	}
    }
  if(![fm removeFileAtPath: path handler: nil])
    NSRunAlertPanel(@"Could Not Delete Contact",
		    [NSString stringWithFormat:
				@"'@' could not be deleted from group %@.\n"
			      @"Insufficient access rights?",
			      [contact screenName], group],
		    @"OK", nil, nil, nil);
}

- (void) deleteContact: (Contact *) contact
{
  if(!NSRunAlertPanel(@"Delete contact?",
		      [NSString stringWithFormat: @"Do you really want to "
				@"delete %@?", [contact screenName]],
		      @"Yes", @"No", nil, nil))
    return;
  
  NSString *path;

  /* First remove from all groups */
  
  NSArray *groups = [self groupNames];
  int i;
  for(i=0; i<[groups count]; i++)
      [self deleteContact: contact
	    fromGroup: [groups objectAtIndex: i]
	    askForConfirmation: NO];
  
  /* Then remove from "All" */
  path = [abRootPath stringByAppendingPathComponent: [contact fileName]];
  if(![fm removeFileAtPath: path handler: nil])
    NSRunAlertPanel(@"Could Not Delete Contact",
		    [NSString stringWithFormat: @"'@' could not be deleted.\n"
			      @"Insufficient access rights?",
			      [contact screenName]],
		    @"OK", nil, nil, nil);
}

- (void) invalidateCache
{
  [allCache release];
  allCache = nil;
}

- (void) selectContactWithId: (int) contactId
{
  [groupsBrowser selectRow: 0 inColumn: 0]; // make sure "All" is selected
  NSArray *contacts = [self contactsInGroup: nil];
  int i;
  for(i=0; i<[contacts count]; i++)
    if([[contacts objectAtIndex: i] getId] == contactId)
      {
	[groupsBrowser selectRow: i inColumn: 1];
	[contactView setContact: [contacts objectAtIndex: i]];
	return;
      }
}

/*
  Actions
*/

- (void) doToggleContactEditable: (id) sender
{
  if([sender state] == NSOnState)
    [contactView setEditable: YES];
  else
    {
      [contactView setEditable: NO];
      [self invalidateCache];
      [groupsBrowser reloadColumn: 1];
      [self selectContactWithId: [[contactView contact] getId]];
    }
  [clipView scrollToPoint: NSMakePoint(0, [contactView frame].size.height)];
}

- (void) doCreateContact: (id) sender
{
  Contact *c;
  c = [[Contact alloc]
	initWithDictionary: [NSMutableDictionary dictionaryWithCapacity: 5]
	id: [self nextValidContactId]];
  [contactView setContact: c];
  [contactView setEditable: YES];
  [editButton setState: NSOnState];
}

- (void) doDeleteContact: (id) sender
{
  NSCell *cell = [groupsBrowser selectedCell];
  if(cell != [groupsBrowser selectedCellInColumn: 1])
    return;

  NSString *group = [[groupsBrowser selectedCellInColumn: 0] stringValue];
  NSArray *contacts = [self contactsInGroup: group];
  int row = [groupsBrowser selectedRowInColumn: 1];
  Contact *contact = [contacts objectAtIndex: row];

  if([group isEqualToString: @"All"])
    {
      [self deleteContact: contact];
      [self invalidateCache];
    }
  else
    [self deleteContact: contact fromGroup: group askForConfirmation: YES];
  
  [groupsBrowser reloadColumn: 1];

  if(row != 0)
    row--;
  [groupsBrowser selectRow: row inColumn: 1];
  contacts = [self contactsInGroup: group];
  if(![contacts count])
    contact = nil;
  else
    contact = [contacts objectAtIndex: row];
  [contactView setContact: contact];
  [clipView scrollToPoint: NSMakePoint(0, [contactView frame].size.height)];
}

- (void) doImportContact: (id) sender
{
  NSOpenPanel *p = [NSOpenPanel openPanel];
  [p setCanChooseDirectories: NO];
  [p setAllowsMultipleSelection: NO];
  [p runModalForTypes: [NSArray arrayWithObjects: @"vcf", nil]];

  NSArray *fnames = [p filenames];
  if(!fnames) return;

  Contact *c = nil;
  [VCFParser takeContactsFromFile: [fnames objectAtIndex: 0]];
  
  while(YES)
    {
      int loaded = 0;
      int nextId = [self nextValidContactId];
      Contact *newC = [[[Contact alloc] initWithId: nextId] autorelease];
      if(![VCFParser nextContact: newC])
	{
	  NSLog(@"Done, loaded %d contacts\n", loaded);
	  break;
	}
      loaded++;

      NSEnumerator *e = [[self contactsInGroup: nil] objectEnumerator];
      id other;
      int retval;

      retval = 0; // insert anyway
      while((other = [e nextObject]))
	{
	  if([[other screenName] isEqualToString: [c screenName]])
	    {
	      NSString *fmt; 
	      fmt =
		[NSString stringWithFormat:
			    @"Trying to import contact named '%@',\n"
			  @"which already exists in the database.",
			  [c screenName]];
	      retval = NSRunAlertPanel(@"Existing contact?",
				       fmt, @"Replace", @"Insert anyway",
				       @"Don't insert", nil);
	      break;
	    }
	}
      if(retval == 1) // replace
	{
	  [self deleteContact: other
		fromGroup: nil
		askForConfirmation: NO];
	  [newC writeToFileInDirectory: abRootPath];
	  c = newC;
	}
      else if(retval == 0) // insert anyway
	{
	  [newC writeToFileInDirectory: abRootPath];
	  c = newC;
	}
      else if(retval == -1) // don't insert; continue reading
	continue;
    }

  [self invalidateCache];

  [groupsBrowser reloadColumn: 1];

  NSString *group = [[groupsBrowser selectedCellInColumn: 0] stringValue];
  NSArray *contacts = [self contactsInGroup: group];
  [groupsBrowser selectRow: 0 inColumn: 1];
  if(c)
    {
      [contactView setContact: c];
      [clipView scrollToPoint: NSMakePoint(0, [contactView frame].size.height)];
    }
  else
    [contactView setContact: [contacts objectAtIndex: 0]];
}

- (void) doCreateGroup: (id)sender
{
  NSString *name = @"New Group";
  NSString *pathname;
  int i = 1;
  
  while(YES)
    {
      pathname = [abRootPath stringByAppendingPathComponent: name];
      if(![fm fileExistsAtPath: pathname])
	break;
      name = [NSString stringWithFormat: @"New Group %d", i++];
    }

  if(![fm createDirectoryAtPath: pathname
	  attributes: nil])
    NSRunAlertPanel(@"Could not create group",
		    [NSString stringWithFormat:
				@"The group %@ could not be created.\n"
			      @"Insufficient access rights for %@?",
			      name, abRootPath],
		    @"OK", nil, nil, nil);
  [groupsBrowser reloadColumn: 0];
}

- (void) doDeleteGroup: (id) sender
{
  NSCell *cell = [groupsBrowser selectedCell];
  if(cell != [groupsBrowser selectedCellInColumn: 0])
    return;
  NSString *group = [cell stringValue];
  if([group isEqualToString: @"All"])
    {
      NSRunAlertPanel(@"Could Not Delete Group",
		      @"The group 'All' cannot be deleted.",
		      @"OK", nil, nil, nil);
      return;
    }

  int retval = NSRunAlertPanel(@"Delete Group?",
			       [NSString stringWithFormat:
					   @"Do you really want to "
					 @"delete '%@'?",
					 group],
			       @"Yes", @"No", nil, nil);
  if(!retval) return;

  NSString *fullpath = [abRootPath stringByAppendingPathComponent: group];
  if(![fm removeFileAtPath: fullpath handler: nil])
    NSRunAlertPanel(@"Could Not Delete Group",
		    [NSString stringWithFormat: @"'@' could not be deleted.\n"
			      @"Insufficient access rights?", group],
		    @"OK", nil, nil, nil);
  else
    {
      if(cell == lastCell)
	lastCell = nil;
      [groupsBrowser reloadColumn: 0];
      [groupsBrowser selectRow: 0 inColumn: 0];
    }
}



/*
  Browser delegate methods
*/

- (int) browser: (NSBrowser*) sender
numberOfRowsInColumn: (int) column
{
  NSArray *groupnames = [self groupNames];

  if(column == 0)
    {
      return [groupnames count]+1;
    }
  else
    {
      id cell = [sender selectedCellInColumn: 0];
      NSString *group = [cell stringValue];

      [lastCell setEditable: NO];
      if(cell == lastCell && ![lastCellLastValue isEqualToString: group])
	{
	  if([group isEqualToString: @"All"])
	    {
	      [cell setStringValue: lastCellLastValue];
	      NSRunAlertPanel(@"Disallowed",
			      @"You cannot rename a group to \"All\", since\n"
			      @"that name is reserved by the system.",
			      @"OK", nil, nil, nil);
	    }
	  else if([groupnames containsObject: group])
	    {
	      [cell setStringValue: lastCellLastValue];
	      NSRunAlertPanel(@"Disallowed",
			      [NSString stringWithFormat:
					  @"You cannot rename this group "
					@"to \"%@\",\n"
					@"since a group of that name "
					@"already exists.", group],
			      @"OK", nil, nil, nil);
	    }
	  else
	    [self renameGroup: lastCellLastValue to: group];
	  [lastCell setEditable: NO];
	}

      group = [cell stringValue];
      if(![group isEqualToString: @"All"])
	{
	  if([cell isEditable])
	    [cell setEditable: NO];
	  else
	    [cell setEditable: YES];

	  [lastCell release];
	  lastCell = [cell retain];
	  [lastCellLastValue release];
	  lastCellLastValue = [[cell stringValue] retain];
	}

      return [[self contactsInGroup: group] count];
    }

  return 0;
}

- (void) browser: (NSBrowser*) sender
 willDisplayCell: (id) cell
	   atRow: (int) row
	  column: (int) column
{
  if(column == 0)
    {
      if(row == 0)
	[cell setStringValue: @"All"];
      else
	{
	  [cell setStringValue: [[self groupNames] objectAtIndex: row-1]];
	  [cell setEditable: NO];
	}
    }
  else
    {
      NSString *group = [[sender selectedCellInColumn: 0] stringValue];
      
      Contact *contact = [[self contactsInGroup: group] objectAtIndex: row];
      [cell setStringValue: [contact screenName]];
      [cell setLeaf: YES];
    }      
}

- (void) browserAction: (id) sender
{
  NSCell *cell = [sender selectedCell];
  NSString *group = [[sender selectedCellInColumn: 0] stringValue];

  if([contactView isEditable])
    {
      [contactView setContact: nil];
      [contactView setEditable: NO];
      [self invalidateCache];
    }

  if(cell == [sender selectedCellInColumn: 1])
    {
      int row = [sender selectedRowInColumn: 1];

      Contact *contact = [[self contactsInGroup: group] objectAtIndex: row];
      [self displayContact: contact];
    }
  else
    [self displayContact: nil];
}

@end
