// NSMatrix+DragDrop.m (this is -*- ObjC -*-)
// 
// \author: Bjrn Giesler <giesler@ira.uka.de>
// 
// 
// 
// $Author: bjoern $
// $Locker:  $
// $Revision: 1.1.1.1 $
// $Date: 2003/08/28 15:54:43 $

#include <AppKit/AppKit.h>
#include "Controller.h"
#include "DragDropMatrix.h"

NSBrowserCell *oldCell = nil;
NSRect oldFrame;
int contactRow = -1;
int groupRow = -1;

@implementation NSBrowserCell (DragAndDrop)
- (NSImage *) getImageOfContents: (NSSize) a_frame
{
  NSImage *image = nil;
  NSRect textFrame;
  
  textFrame.size = a_frame;
  textFrame.origin = NSZeroPoint;
  
  image = [[NSImage alloc] initWithSize: textFrame.size];
  [image setBackgroundColor:[NSColor clearColor]];
  
  [image lockFocus];
  [self drawInteriorWithFrame: textFrame inView: [NSView focusView]];
  [image unlockFocus];

  return [image autorelease];
}
@end

@implementation DragDropMatrix
- (BOOL) acceptsFirstMouse: (NSEvent *) theEvent
{
  return YES;
}

- (void) copyToPasteboard: (NSPasteboard *) a_pboard
{
  NSString *title = [[self selectedCell] title];
  NSAttributedString *attTitle = [[self selectedCell] attributedStringValue];
  
  [a_pboard declareTypes: [NSArray arrayWithObjects: NSRTFPboardType,
				   NSStringPboardType, nil]
	    owner: self]; 
  
  [a_pboard setData: [attTitle RTFFromRange: NSMakeRange(0, [attTitle length])
			       documentAttributes: nil]
	    forType: NSRTFPboardType];
  [a_pboard setString: title forType: NSStringPboardType];
}

- (void) mouseDown: (NSEvent *) event
{
  int row = -1, column = -1;

  [self registerForDraggedTypes: [NSArray arrayWithObjects: NSStringPboardType,
					  nil]];
  
  if([self getRow: &row
	   column: &column
	   forPoint: [self convertPoint:[event locationInWindow]
			   fromView: nil]] &&
     [[self cellAtRow:row column: column] isEnabled])
    {
      [self selectCellAtRow: row column: column];
      [self sendAction];
    }
  else
    {
      [super mouseDown: event];
    }
}

- (void) mouseDragged: (NSEvent *) event
{
  NSPasteboard *pboard;
  NSImage *image = nil;
  NSImage *translucent = nil;
  NSPoint dragPoint = NSZeroPoint;
  NSPoint cellOrigin;
  int width=0, height=0;
  NSCell *cell;

  cell = [self selectedCell];
  if(![((NSBrowserCell*)cell) isLeaf]) // can only drag leaf cells
    return;
  
  pboard = [NSPasteboard pasteboardWithName: NSDragPboard];
  [self copyToPasteboard: pboard];
  
  image = [[self selectedCell] getImageOfContents: [self cellSize]]; 
  dragPoint = [self convertPoint: [event locationInWindow]
		    fromView: nil];
  contactRow = [self selectedRow];

  cellOrigin = [self cellFrameAtRow: [self selectedRow]
		     column: [self selectedColumn]].origin;

  width = [image size].width;
  height = [image size].height;

  dragPoint.x -= width * (dragPoint.x - cellOrigin.x) / width;
  dragPoint.y += height * (cellOrigin.y + height - dragPoint.y) / height;

  translucent = [[[NSImage alloc] initWithSize: [image size]]
		  autorelease];
  [translucent lockFocus];
  [image dissolveToPoint: NSZeroPoint fraction: 0.5];
  [translucent unlockFocus];

  [self dragImage: translucent
	at: dragPoint
	offset: NSMakeSize(0,0)
	event: event
	pasteboard: pboard
	source: self
	slideBack: YES];
}

- (NSDragOperation) draggingSourceOperationMaskForLocal: (BOOL) local
{
  return NSDragOperationLink;
}

- (void) draggingExited:  (id<NSDraggingInfo>) sender
{
  if(oldCell)
    {
      [self lockFocus];
      [oldCell drawWithFrame: oldFrame inView: self];
      [oldCell release];
      oldCell = nil;
      [_window flushWindow];
      [self unlockFocus];
    }
}

- (NSDragOperation) draggingUpdated: (id<NSDraggingInfo>) sender
{
  int row, column;
  NSRect frame;
  NSBrowserCell *cell;

  NSPoint p = [self convertPoint: [sender draggingLocation]
		    fromView: nil];

  if(![self getRow: &row column: &column forPoint: p] ||
     self == [sender draggingSource] ||
     row == 0)
    {
      if(oldCell)
	{
	  [self lockFocus];
	  [oldCell drawWithFrame: oldFrame inView: self];
	  [oldCell release];
	  oldCell = nil;
	  [_window flushWindow];
	  [self unlockFocus];
	}

      groupRow = -1;
      return NSDragOperationNone;
    }

  groupRow = row;
  cell = [self cellAtRow: row column: column];
  if(cell == [self selectedCell]) // can't drag to current group
    return NSDragOperationNone;
  
  frame = [self cellFrameAtRow: row column: column];

  [self lockFocus];
  if(cell != oldCell)
    {
      if(oldCell)
	{
	  NSRectFill(oldFrame);
	  [oldCell drawWithFrame: oldFrame inView: self];
	  [oldCell release];
	}
      oldCell = [cell retain];
      oldFrame = frame;
    }
  
  [[NSColor darkGrayColor] set];
  NSFrameRect(frame);
  [_window flushWindow];
  [self unlockFocus];

  return NSDragOperationLink;
}

- (BOOL) prepareForDragOperation: (id<NSDraggingInfo>) sender
{
  return YES;
}

- (BOOL) performDragOperation: (id<NSDraggingInfo>) sender
{
  if(oldCell)
    {
      [self lockFocus];
      [oldCell drawWithFrame: oldFrame inView: self];
      [oldCell release];
      oldCell = nil;
      [_window flushWindow];
      [self unlockFocus];
    }

  //                     clipview   scrollview browserview
  NSBrowser *v = [[[self superview] superview] superview];
  id deleg = [v delegate];
  if([deleg respondsToSelector: @selector(linkContactAtRow:toGroupAtRow:)])
    [deleg linkContactAtRow: contactRow toGroupAtRow: groupRow];
  return YES;
}

@end
