#include "VCFParser.h"

@implementation NSString (QuotedPrintable)
- (unsigned long) hexLongValue
{
  unsigned long val = 0;
  NSString *str =
    [[self stringByTrimmingCharactersInSet: [NSCharacterSet whitespaceCharacterSet]]
      lowercaseString];
  NSString *hexchars = @"0123456789abcdef";
  int i;

  for(i=0; i<[str length]; i++)
    {
      NSRange r = NSMakeRange(i, 1);
      r = [hexchars rangeOfString: [str substringWithRange: r]];
      assert(r.location != NSNotFound);
      val <<= 4;
      val |= r.location;
    }

  return val;
}

- (NSString*) stringByQuotedPrintableDecoding
{
  int i;
  
  NSMutableString *str = [NSMutableString stringWithCapacity: [self length]];
  for(i=0; i<[self length]; i++)
    {
      NSRange r = NSMakeRange(i, 1);
      NSString *s = [self substringWithRange: r];
      if([s isEqualToString: @"="])
	{
	  unsigned char c;
	  
	  r = NSMakeRange(i+1, 2);
	  NSString *hex = [self substringWithRange: r];
	  c = (unsigned char)[hex hexLongValue];

	  [str appendString: [NSString stringWithFormat: @"%c", c]];
	  i+=2;
	}
      else
	[str appendString: s];
    }

  str = [NSString stringWithUTF8String: [str cString]];
  return str;
}
@end

@interface NSArray (BGStuff)
- (NSString*) restOfStringStartingWith: (NSString*) start;
@end

@implementation NSArray (BGStuff)
- (NSString*) restOfStringStartingWith: (NSString*) start
{
  NSEnumerator *e = [self objectEnumerator];
  id obj;
  
  while((obj = [e nextObject]))
    {
      if(![obj isKindOfClass: [NSString class]])
	continue;
      if([obj length] < [start length])
	continue;
      if([[obj substringToIndex: [start length]]
	   isEqualToString: start])
	return [obj substringFromIndex: [start length]];
    }
  return nil;
}
@end

NSData *base64Decode(NSString* nsstr)
{
  char *b64chars =
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
  int i, j;
  unsigned char *bytes; const char *str;

  //assert((int)([nsstr length]/4) == ((float)[nsstr length])/4.0);
  str = [nsstr lossyCString];
  
  bytes = (unsigned char*)calloc(([nsstr length]/4)*3, 1);
  
  for(i=0, j=0; i < strlen(str); i+= 4, j+=3)
    {
      unsigned char s1, s2, s3, s4;
      u_int32_t v;

      while(!strchr(b64chars, str[i]) && i<strlen(str))
	i++;
      if(i>=strlen(str))
	{
	  NSLog(@"Error: No good characters in string\n");
	  break;
	}
      
      s1 = strchr(b64chars, str[i]) - b64chars;
      s2 = strchr(b64chars, str[i+1]) - b64chars;
      s3 = strchr(b64chars, str[i+2]) - b64chars;
      s4 = strchr(b64chars, str[i+3]) - b64chars;
      v = (s1 << 18) | (s2 << 12) | (s3 << 6) | s4;
      bytes[j] = (v & 0xff0000) >> 16;
      bytes[j+1] = (v & 0xff00) >> 8;
      bytes[j+2] = v & 0xff;
    }
  return [NSData dataWithBytes: bytes length: ([nsstr length]/4)*3];
}
      

static NSArray *knownItems;
static NSString *string;
int idx;

@implementation VCFParser
+ (void) initialize
{
  knownItems =
    [[NSArray alloc]
      initWithObjects: @"adr", @"org", @"n", @"agent",
      @"logo", @"photo", @"label", @"fn", @"title", @"sound",
      @"version", @"tel", @"email", @"tz", @"geo", @"note",
      @"url", @"bday", @"role", @"rev", @"uid", @"key", @"mailer",
      nil];
}

+ (void) takeContactsFromFile: (NSString*) filename
{
  NSString *s = [NSString stringWithContentsOfFile: filename];
  [self takeContactsFromString: s];
}

+ (void) takeContactsFromString: (NSString*) str
{
  if(string) [string release];
  string = [[NSString alloc] initWithString: str];
  idx = 0;
}

+ (BOOL) nextContact: (Contact*) contact
{
  NSString *str = [string substringFromIndex: idx];
  
  NSArray *lines = [str componentsSeparatedByString: @"\n"];
  NSLog(@"%d lines in file\n", [lines count]);
  
  NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithCapacity: 5];
  
  int i = 0;
  while(i < [lines count])
    {
      NSArray *keyblock, *valueblock;
      BOOL retval;
      int oldIndex, newIndex;

      oldIndex = i;
      retval = [self parseLine: i
		     fromArray: lines
		     upToLine: &i
		     intoKeyBlock: &keyblock
		     valueBlock: &valueblock];
      newIndex = i;
      while(oldIndex < newIndex)
	idx += [[lines objectAtIndex: oldIndex++] length] + 1;
      
      if(retval)
	{
	  if([keyblock containsObject: @"end"]) // done!
	  {
	    [contact setDictionary: dict];
	    return YES;
	  }
	     
	  if(![keyblock containsObject: @"begin"] &&
	     ![keyblock containsObject: @"end"] &&
	     ![keyblock containsObject: @"version"])
	    [self integrateKeyBlock: keyblock
		  valueBlock: valueblock
		  intoDictionary: dict];
	  }
      }
  
  return NO;
}

+ (BOOL) parseLine: (int) line
	 fromArray: (NSArray*) arr
	  upToLine: (int*) retLine
      intoKeyBlock: (NSArray**) k
	valueBlock: (NSArray**) v
{
  NSString *str, *keyblock, *value;
  NSCharacterSet *wsp = [NSCharacterSet whitespaceAndNewlineCharacterSet];
  
  *retLine = line;
  
  str = [[arr objectAtIndex: (*retLine)++]
	    stringByTrimmingCharactersInSet: wsp];
  if(![str length]) return NO;
  while(*retLine < [arr count] &&
	[[arr objectAtIndex: *retLine] length] &&
	[[[arr objectAtIndex: *retLine]
	   substringWithRange: NSMakeRange(0, 1)]
	  isEqualToString: @" "])
    {
      NSString *str2 = [[arr objectAtIndex: (*retLine)++]
			 stringByTrimmingCharactersInSet: wsp];
      str = [str stringByAppendingString: @" "];
      str = [str stringByAppendingString: str2];
    }
		 
  NSRange r = [str rangeOfString: @":"];
  if(r.location == NSNotFound)
    {
      NSLog(@"Syntax error in line %d!\n", line);
      return NO;
    }
  
  keyblock = [str substringToIndex: r.location];
  *k = [[keyblock lowercaseString] componentsSeparatedByString: @";"];
  
  value = [str substringFromIndex: r.location+r.length];
  if([value isEqualToString: @"="]) // escape to next line
    {
      value = [[arr objectAtIndex: (*retLine)++]
		stringByTrimmingCharactersInSet: wsp];
      *v = [[value stringByQuotedPrintableDecoding]
	     componentsSeparatedByString: @";"];
    }
  else if([*k containsObject: @"encoding=base64"])
    {
      NSString *format = [*k restOfStringStartingWith: @"type="];
      NSData *data = base64Decode(value);
      NSString *filename = [NSString stringWithFormat: @"/tmp/img.%@",
				     format];

      NSLog(@"Saving to %@\n", filename);
      assert(![[NSFileManager defaultManager] fileExistsAtPath: filename]);
      if(![data writeToFile: filename atomically: NO])
	{
	  NSLog(@"Couldn't write %@\n", filename);
	  return NO;
	}

      NSImage *img = [[[NSImage alloc]
			initWithContentsOfFile: filename]
		       autorelease];
      [[NSFileManager defaultManager]
	removeFileAtPath: filename handler: nil];
      if(!img)
	return NO;
      
      *v = [NSArray arrayWithObjects: [img TIFFRepresentation], nil];
    }
  else
    *v = [[value stringByQuotedPrintableDecoding]
	   componentsSeparatedByString: @";"];

  return YES;
}

+ (void) integrateKeyBlock: (NSArray*) k
		valueBlock: (NSArray*) v
	    intoDictionary: (NSMutableDictionary*) dict
{
  int i;
  NSMutableArray *params; NSString* key;

  key = [k objectAtIndex: 0];
  if(![knownItems containsObject: key])
    {
      NSLog(@"%@ not in knownItems\n", key);
      return;
    }

  params = [NSMutableArray arrayWithCapacity: [k count]-1];
  for(i=0; i<[k count]; i++)
    [params addObject: [k objectAtIndex: i]];
  
  NSMutableDictionary *valDict =
    [NSMutableDictionary dictionaryWithObjectsAndKeys: v, @"Values", nil];
  if([params count])
    [valDict setObject: params forKey: @"Parameters"];

  [valDict setObject: [[NSDate date] description] forKey: @"Timestamp"];
  
  NSMutableArray *values = [dict objectForKey: key];
  if(!values)
    {
      values = [NSMutableArray arrayWithObjects: valDict, nil];
      [dict setObject: values forKey: key];
    }
  else
    [values addObject: valDict];
}

@end
